package internal

import (
	"context"
	"fmt"
	"gitea.cdlsxd.cn/sdk/plugin/proto"
	"github.com/carlmjohnson/requests"
	"plugins/union_pay_cpn/internal/po"
)

// 插件通信信息,若不对应则会报错panic
const (
	Tag         = "union_pay_cpn"
	Version     = 1
	CookieKey   = "union_pay_cpn"
	CookieValue = "union_pay_cpn"
)

const (
	orderMethod    = "/upapi/mkt/cpn/couponAcquire/v1"
	orderBizMethod = "mkt.cpn.couponAcquire.v1"
)

const (
	queryMethod    = "/upapi/mkt/cpn/couponAcqQuery/v1"
	queryBizMethod = "mkt.cpn.couponAcqQuery.v1"
)

const notifyBizMethod = "mkt.CpnStateUpdtNotify"

type UnionPayCpnService struct{}

func (p *UnionPayCpnService) Order(ctx context.Context, request *proto.OrderRequest) (*proto.OrderResponse, error) {
	c, err := transConfig(request.Config)
	if err != nil {
		return nil, err
	}
	uv, err := c.orderReq(request)
	if err != nil {
		return nil, err
	}
	if err = uv.Validate(); err != nil {
		return nil, err
	}
	var response po.OrderResp
	url := fmt.Sprintf("%s%s", c.BaseUri, orderMethod)
	err = requests.URL(url).Headers(headers(c, uv, orderBizMethod)).BodyJSON(uv).ToJSON(&response).Fetch(ctx)
	if err != nil {
		return nil, fmt.Errorf("请求异常，msg:" + err.Error())
	}

	return orderResp(request, response), nil
}

func (p *UnionPayCpnService) Query(ctx context.Context, request *proto.QueryRequest) (*proto.QueryResponse, error) {
	conf, err := transConfig(request.Config)
	if err != nil {
		return nil, err
	}
	uv := queryReq(request, conf.ChNlId)
	if err = uv.Validate(); err != nil {
		return nil, err
	}
	var response po.QueryResp
	url := fmt.Sprintf("%s%s", conf.BaseUri, queryMethod)
	err = requests.URL(url).Headers(headers(conf, uv, queryBizMethod)).BodyJSON(uv).ToJSON(&response).Fetch(ctx)
	if err != nil {
		return nil, fmt.Errorf("请求异常，msg:" + err.Error())
	}

	return queryResp(request, response), nil
}

func (p *UnionPayCpnService) Notify(_ context.Context, request *proto.NotifyRequest) (*proto.NotifyResponse, error) {
	uv := notifyReq(request)
	if err := uv.Validate(); err != nil {
		return nil, err
	}

	conf, err := transConfig(request.Config)
	if err != nil {
		return nil, err
	}

	if err = verify(conf, uv, notifyBizMethod); err != nil {
		return nil, err
	}

	return notifyResp(request, uv), nil
}
